@@ -11,10 +11,36 @@ SEXP SP_PREFIX(Polygon_c)(SEXP coords, SEXP n, SEXP ihole) {
     int pc=0, rev=FALSE;
     int i, ii, nn=INTEGER_POINTER(n)[0];
     SEXP valid;
+    SEXP coords1, dim1;
+
+    for (i=0; i<nn; i++) {
+       if(!R_FINITE(NUMERIC_POINTER(coords)[i]))
+           error("non-finite x coordinate");
+       if(!R_FINITE(NUMERIC_POINTER(coords)[i+nn]))
+           error("non-finite y coordinate");
+    }
+
+// 140406 ring closure attempt
+    if (NUMERIC_POINTER(coords)[0] != NUMERIC_POINTER(coords)[nn-1]
+        || NUMERIC_POINTER(coords)[nn] != NUMERIC_POINTER(coords)[(2*nn)-1]) {
+        PROTECT(coords1 = NEW_NUMERIC((nn+1)*2)); pc++;
+        PROTECT(dim1 = NEW_INTEGER(2)); pc++;
+        for (i=0; i<nn; i++) {
+            NUMERIC_POINTER(coords1)[i] = NUMERIC_POINTER(coords)[i];
+            NUMERIC_POINTER(coords1)[i+(nn+1)] = NUMERIC_POINTER(coords)[i+nn];
+        }
+        NUMERIC_POINTER(coords1)[nn] = NUMERIC_POINTER(coords)[0];
+        NUMERIC_POINTER(coords1)[(2*(nn+1))-1] = NUMERIC_POINTER(coords)[nn];
+        nn++;
+        INTEGER_POINTER(dim1)[0] = nn;
+        INTEGER_POINTER(dim1)[1] = 2;
+        setAttrib(coords1, R_DimSymbol, dim1);
+        coords = coords1;
+    }
 
     SP_PREFIX(spRFindCG_c)(n, coords, &xc, &yc, &area);
-    if (abs(area) < DOUBLE_EPS) {
-        if (!R_FINITE(xc) || !R_FINITE(xc)) {
+    if (fabs(area) < DOUBLE_EPS) {
+        if (!R_FINITE(xc) || !R_FINITE(yc)) {
             if (nn == 1) {
                 xc = NUMERIC_POINTER(coords)[0];
                 yc = NUMERIC_POINTER(coords)[1];
@@ -75,6 +101,7 @@ SEXP SP_PREFIX(Polygon_c)(SEXP coords, SEXP n, SEXP ihole) {
        }
     }
     
+
     SET_SLOT(SPans, install("coords"), coords);
 
     PROTECT(labpt = NEW_NUMERIC(2)); pc++;
@@ -90,6 +117,7 @@ SEXP SP_PREFIX(Polygon_c)(SEXP coords, SEXP n, SEXP ihole) {
     SET_SLOT(SPans, install("ringDir"), ringDir);
 
     PROTECT(valid = SP_PREFIX(Polygon_validate_c)(SPans)); pc++;
+
     if (!isLogical(valid)) {
         UNPROTECT(pc);
         if (isString(valid)) error(CHAR(STRING_ELT(valid, 0)));
@@ -408,7 +436,7 @@ void SP_PREFIX(spRFindCG_c)( SEXP n, SEXP coords, double *xc, double *yc,
 	tPointd CG;
 	double Areasum2;
 	nn = INTEGER_POINTER(n)[0];
-	P = (tPointd *) R_alloc(nn, sizeof(tPointd));
+	P = (tPointd *) R_alloc((size_t) nn, sizeof(tPointd));
 	for (i=0; i<nn; i++) {
 		P[i][0] = NUMERIC_POINTER(coords)[i];
 		P[i][1] = NUMERIC_POINTER(coords)[i+nn];
@@ -475,6 +503,8 @@ SEXP SP_PREFIX(comment2comm)(SEXP obj) {
     }
 
     nc = length(STRING_ELT(comment, 0));
+    if (nc < 1) error("comment2comm: empty string comment");
+
     buf = (char *) R_alloc((size_t) (nc+1), sizeof(char));
 
     strcpy(buf, CHAR(STRING_ELT(comment, 0)));
@@ -548,7 +578,7 @@ void SP_PREFIX(comm2comment)(char *buf, int bufsiz, int *comm, int nps) {
     char cbuf[15];
     int i, nc, nc1;
 
-    nc = ceil(log10(nps)+1)+1;
+    nc = (int) (ceil(log10(nps)+1.0)+1.0);
     nc1 = (nc*nps)+1;
     if (bufsiz < nc1) error("comm2comment: buffer overflow");
 
