@@ -1,3 +1,5 @@
+#include <string.h>
+
 #define USING_R 1
 #include "S.h" 
 
@@ -9,14 +11,35 @@
 
 #include "sp.h"
 
-SEXP sp_zerodist(SEXP pp, SEXP pncol, SEXP zero, SEXP lonlat) {
-	unsigned int i, j, k, ncol, nrow, nzero = 0, *which = NULL, ll;
-	double **x, *xi, *xj, d, dist, zerodist2;
+int is_zero(double *xi, double *xj, int ncol, int ll, double zerodist2,
+		int cmp) {
+	int k;
+	double d, dist;
+
+	if (zerodist2 <= 0.0) /* check bitwise equality */
+		return (memcmp(xi, xj, ncol * sizeof(double)) == 0); 
+	/* bit-wise difference: compute dist, and compare to zerodist2 */
+	if (ll) {
+		sp_gcdist(xi, xj, xi+1, xj+1, &d);
+		dist = d * d;
+	} else {
+		for (k = 0, dist = 0.0; k < ncol; k++) {
+			d = (xi[k] - xj[k]);
+			dist += d * d;
+		}
+	}
+	return(dist <= zerodist2);
+}
+
+SEXP sp_zerodist(SEXP pp, SEXP pncol, SEXP zero, SEXP lonlat, SEXP mcmp) {
+	unsigned int i, j, ncol, nrow, nzero = 0, *which = NULL, ll, cmp;
+	double **x, zerodist2;
 	SEXP ret = NULL;
 
 	S_EVALUATOR
 	ncol = INTEGER_POINTER(pncol)[0];
 	ll = INTEGER_POINTER(lonlat)[0];
+	cmp = INTEGER_POINTER(mcmp)[0];
 	if (ll && ncol != 2)
 		error("for longlat data, coordinates should be two-dimensional");
 	nrow = LENGTH(pp)/ncol;
@@ -29,19 +52,8 @@ SEXP sp_zerodist(SEXP pp, SEXP pncol, SEXP zero, SEXP lonlat) {
 		x[i] = &(NUMERIC_POINTER(pp)[i*ncol]);
 
 	for (i = 0; i < nrow; i++) {
-		xi = x[i];
 		for (j = 0; j < i; j++) {
-			xj = x[j];
-			if (ll) {
-				sp_gcdist(xi, xj, xi+1, xj+1, &d);
-				dist = d * d;
-			} else {
-				for (k = 0, dist = 0.0; k < ncol; k++) {
-					d = (xi[k] - xj[k]);
-					dist += d * d;
-				}
-			}
-			if (dist <= zerodist2) {
+			if (is_zero(x[i], x[j], ncol, ll, zerodist2, cmp)) {
 				which = (unsigned int *) realloc(which, (size_t) (nzero+2) * sizeof(unsigned int));
 				if (which == NULL)
 					error("could not allocate vector of %u bytes in zerodist",
@@ -63,14 +75,15 @@ SEXP sp_zerodist(SEXP pp, SEXP pncol, SEXP zero, SEXP lonlat) {
 	return(ret);
 }
 
-SEXP sp_duplicates(SEXP pp, SEXP pncol, SEXP zero, SEXP lonlat) {
-	unsigned int i, j, k, ncol, nrow, ll, next;
-	double **x, *xi, *xj, d, dist, zerodist2;
+SEXP sp_duplicates(SEXP pp, SEXP pncol, SEXP zero, SEXP lonlat, SEXP mcmp) {
+	unsigned int i, j, k, ncol, nrow, ll, next, cmp;
+	double **x, zerodist2;
 	SEXP ret = NULL;
 
 	S_EVALUATOR
 	ncol = INTEGER_POINTER(pncol)[0];
 	ll = INTEGER_POINTER(lonlat)[0];
+	cmp = INTEGER_POINTER(mcmp)[0];
 	if (ll && ncol != 2)
 		error("for longlat data, coordinates should be two-dimensional");
 	nrow = LENGTH(pp)/ncol;
@@ -86,22 +99,12 @@ SEXP sp_duplicates(SEXP pp, SEXP pncol, SEXP zero, SEXP lonlat) {
 	if (nrow > 0)
 		INTEGER_POINTER(ret)[0] = 0;
 	for (i = 1; i < nrow; i++) {
-		xi = x[i];
 		INTEGER_POINTER(ret)[i] = i;
 		next = 0;
 		for (j = 0; next == 0 && j < i; j++) { /* find match */
 			if (INTEGER_POINTER(ret)[j] == j) { /* this is a new point */
-				xj = x[j];
-				if (ll) {
-					sp_gcdist(xi, xj, xi+1, xj+1, &d);
-					dist = d * d;
-				} else {
-					for (k = 0, dist = 0.0; k < ncol; k++) {
-						d = (xi[k] - xj[k]);
-						dist += d * d;
-					}
-				}
-				if (dist <= zerodist2) { /* match! */
+				if (is_zero(x[i], x[j], ncol, ll, zerodist2, cmp)) { 
+					/* match: */
 					INTEGER_POINTER(ret)[i] = j;
 					next = 1; /* break for loop */
 				}
